#cython: language_level=3

from obitools3.dms.capi.obiview cimport NUC_SEQUENCE_COLUMN, \
                                        ID_COLUMN, \
                                        DEFINITION_COLUMN, \
                                        QUALITY_COLUMN, \
                                        COUNT_COLUMN

from obitools3.utils cimport str2bytes


cdef class HeaderFormat:
    
    SPECIAL_KEYS = [NUC_SEQUENCE_COLUMN, ID_COLUMN, DEFINITION_COLUMN, QUALITY_COLUMN]
    
    def __init__(self, str format="fasta", list tags=[], bint printNAKeys=False):
        '''
            @param format: 
            @type  format: `str`
            
            @param tags:
            @type  tags: `list` of `bytes`
            
            @param printNAKeys: 
            @type  printNAKeys: `bool` 
        '''
        
        self.tags           = set(tags)
        self.printNaKeys    = printNAKeys
        
        if format=="fasta":
            self.start=b">"
        elif format=="fastq":
            self.start=b"@"
        
        self.headerBufferLength = 1000

        
    def __call__(self, object data):
        cdef bytes header
        cdef list tags = [key for key in data if key not in self.SPECIAL_KEYS]
        cdef set  ktags
        cdef list lines = [b""]
        cdef bytes tagline
        
        
        if self.tags is not None and self.tags:
            ktags = self.tags
        else:
            ktags = set(tags)
            
        for k in ktags:
            if k in tags:
                value = data[k]
                if value is not None or self.printNaKeys:
                    lines.append(k + b"=" + str2bytes(str(data[k])))   #TODO bytes() method on values (str equivalent)
                    
        if len(lines) > 1:
            tagline=b" ".join(lines)
        else:
            tagline=b""
            
        if data[DEFINITION_COLUMN] is not None:
            header = self.start + data[ID_COLUMN] + tagline + b" " + data[DEFINITION_COLUMN]
        else:
            header = self.start + data[ID_COLUMN] + tagline
             
        return header
        
